﻿using Borepin.Page;
using Borepin.PageModel;
using Borepin.Service.Navigation;
using System;
using System.Collections.Generic;
using TinyIoC;

namespace Borepin.Base
{
    public class PageModelLocator
    {
        private static TinyIoCContainer _Container;
        private static Dictionary<Type, Type> _ViewLookUp;

        static PageModelLocator()
        {
            _Container = new TinyIoCContainer();
            _ViewLookUp = new Dictionary<Type, Type>();

            // Register Pages and PageModels
            Register<DashboardPageModel, DashboardPage>();
            Register<LoginPageModel, LoginPage>();

            // Register Services (Services are registered by Singeltons default)
            _Container.Register<INavigationService, NavigationService>();
        }

        public static T Resolve<T>() where T : class
        {
            return _Container.Resolve<T>();
        }

        public static Xamarin.Forms.Page CreatePage(Type pageModelType)
        {
            Type pageType = _ViewLookUp[pageModelType];
            Xamarin.Forms.Page page = (Xamarin.Forms.Page)Activator.CreateInstance(pageType);
            object pageModel = _Container.Resolve(pageModelType);

            page.BindingContext = pageModel;
            return page;
        }

        static void Register<TPageModel, TPage>() where TPageModel : PageModelBase where TPage : Xamarin.Forms.Page
        {
            _ViewLookUp.Add(typeof(TPageModel), typeof(TPage));
            _Container.Register<TPageModel>();
        }
    }
}
